import path from 'node:path';
import { PDF } from '../export/index.js';
import { DBConsultPool } from '../../db/index.js';
import { Subsidiary as SubsidiaryProvider, Casher as CasherProvider, Product as ProductProvider, } from '../../provider/index.js';
import { Product } from '../../service/index.js';
import { AssistantPath, Hash } from '../../util/index.js';
class InstalledProduct {
    constructor() {
        this.allowedFormats = ['PDF'];
        this.title = 'Produtos Instalados por Organização';
        this.fileName = 'produtos-instalados';
    }
    analytic({ subsidiary, casher, products, installedProducts, orgCount }) {
        const data = {};
        const productCount = {};
        installedProducts.forEach((orgData) => {
            const versionOcurrency = {};
            const installedProductsSet = new Set();
            if (!orgData.result)
                return;
            orgData.result.forEach((row) => {
                installedProductsSet.add(row.UPDATERPRODID);
                const pivot = `${row.UPDATERPRODID}_${row.NRVERSAOATUAL}`;
                versionOcurrency[pivot] = versionOcurrency[pivot] || 0;
                versionOcurrency[pivot] += 1;
            });
            [...installedProductsSet].forEach((productId) => {
                productCount[productId] = productCount[productId] || 0;
                productCount[productId] += 1;
            });
            data[orgData.user] = orgData.result.map((row) => {
                const pivot = `${row.UPDATERPRODID}_${row.NRVERSAOATUAL}`;
                const { CDFILIAL, CDCAIXA, UPDATERPRODID, NRVERSAOATUAL } = row;
                const total = (subsidiary[orgData.user] || 0) + (casher[orgData.user] || 0);
                const versionPerc = (versionOcurrency[pivot] * 100) / total;
                return [`${UPDATERPRODID} - ${products[UPDATERPRODID]}`, CDFILIAL, CDCAIXA || '-', `${NRVERSAOATUAL} (${versionPerc.toFixed(2)}%)`];
            });
        });
        const productOcurrency = Object.keys(productCount).map((productId) => {
            const orgsWithProduct = productCount[Number(productId)] || 0;
            return [`${productId} - ${products[productId]}`, `${orgsWithProduct} (${((orgsWithProduct * 100) / orgCount).toFixed(2)}%)`];
        });
        Object.keys(products).forEach((productId) => {
            if (!(productId in productCount)) {
                productOcurrency.push([`${productId} - ${products[productId]}`, '0 (0%)']);
            }
        });
        return { productAndVersion: data, productOcurrency };
    }
    async buildDataSet(connections) {
        const { getAvailableCasher } = CasherProvider;
        const { getAvailableSubsidiary } = SubsidiaryProvider;
        const { getInstalledProducts } = ProductProvider;
        const [casherResult, subsidiaryResult] = await Promise.all([
            Promise.all(DBConsultPool.fetch(connections, {
                method: getAvailableCasher.bind(CasherProvider),
            })),
            Promise.all(DBConsultPool.fetch(connections, {
                method: getAvailableSubsidiary.bind(SubsidiaryProvider),
            })),
        ]);
        const analyseResult = (dbResult, key) => {
            const structure = {};
            dbResult.forEach((row) => {
                const counter = Array.isArray(row.result) ? row.result.pop() : 0;
                if (counter && counter[key]) {
                    structure[row.user] = counter[key];
                }
                else {
                    structure[row.user] = 0;
                }
            });
            return structure;
        };
        const subsidiary = analyseResult(subsidiaryResult, 'AVAILABLE_SUBSIDIARY');
        const casher = analyseResult(casherResult, 'AVAILABLE_CASHER');
        const installedProductsResult = await Promise.all(DBConsultPool.fetch(connections, {
            method: getInstalledProducts.bind(ProductProvider),
        }));
        const errors = installedProductsResult.filter((result) => result.error);
        if (errors.length === connections.length) {
            throw new Error('FAIL_TO_DB_CONNECT_FOR_GET_DATASET');
        }
        const products = await Product.list();
        return {
            errors: errors.length ? errors : null,
            dataSet: this.analytic({
                subsidiary,
                casher,
                installedProducts: installedProductsResult,
                products: products.reduce((acc, cv) => {
                    acc[cv.devopsId] = cv.productName;
                    return acc;
                }, {}),
                orgCount: connections.length,
            }),
        };
    }
    async generate({ parameters, connections, fileName }) {
        return new Promise((resolve, reject) => {
            try {
                const reportPath = AssistantPath.getReportPath();
                this.buildDataSet(connections).then(({ dataSet, errors }) => {
                    const { productOcurrency, productAndVersion } = dataSet;
                    const reportConfig = {
                        info: {
                            Title: this.title,
                        },
                    };
                    let reportFileName = fileName;
                    if (!reportFileName) {
                        const hash = Hash.generate(parameters);
                        reportFileName = `${this.fileName}.${hash}.pdf`;
                    }
                    const reportFile = path.join(reportPath, reportFileName);
                    const PDFDoc = PDF.create(reportConfig, reportFile, true, resolve);
                    const doc = PDFDoc.getDocument();
                    PDFDoc.addTable(['Produto', 'Nº de Organizações'], productOcurrency, { x: 25, y: 60 });
                    Object.keys(productAndVersion).forEach((orgName) => {
                        const orgValue = productAndVersion[orgName];
                        const orgTitle = `Organização: ${orgName}`;
                        doc.font('Helvetica-Bold')
                            .text(orgTitle, 25, doc.y + 20, { align: 'center' })
                            .font('Helvetica');
                        PDFDoc.addTable(['Produto', 'Filial', 'Caixa', 'Versões'], orgValue, { x: 25, y: doc.y + 10 });
                    });
                    if (errors) {
                        doc.font('Helvetica-Bold')
                            .text('* Organizações que ocorreram erro ao conectar ao banco de dados:', 25, doc.y + 20);
                        const orgNames = errors.map(({ user }) => user);
                        doc.font('Helvetica')
                            .text(orgNames.join(', '), 25, doc.y + 20);
                    }
                    doc.end();
                }).catch((err) => {
                    console.error(err.message);
                    reject(err);
                });
            }
            catch (error) {
                reject(error);
            }
        });
    }
}
const instance = new InstalledProduct();
export { instance as InstalledProduct };
export default instance;
