import path from 'node:path';
import fs from 'node:fs';
import { format } from 'date-fns';
import { Crypt, File, Request, Base64, Telegram, AssistantPath } from '../util/index.js';
import { UpdaterVersion, AssistantInstaller } from './index.js';
class AssistantControl {
    async readDataBasePath() {
        const dbPath = AssistantPath.getDataBasePath();
        const versionFolders = await File.readdir(dbPath, { onlyType: 'directory' });
        const getVersionByPath = (pathToAnalise) => {
            const fileName = pathToAnalise.endsWith('.db') ? path.basename(pathToAnalise) : '';
            const versionDir = pathToAnalise.replace(dbPath, '').replace(fileName, '');
            return path.basename(versionDir);
        };
        const handleDbFiles = (dbFiles) => {
            if (Array.isArray(dbFiles) && dbFiles.length) {
                return { version: getVersionByPath(dbFiles[0].path), files: dbFiles };
            }
            return {};
        };
        const readDbFiles = async (dir) => {
            const versionPath = dir.path.endsWith(dir.name) ? dir.path : path.join(dir.path, dir.name);
            const files = await File.readdir(versionPath, { onlyType: 'file' });
            return handleDbFiles(files);
        };
        const databaseFiles = await Promise.all(versionFolders.map(readDbFiles));
        return databaseFiles;
    }
    async getDataBaseList() {
        const dbList = await this.readDataBasePath();
        let response = [];
        dbList.forEach((versionInfo) => {
            if (versionInfo && Array.isArray(versionInfo.files)) {
                response = response.concat(versionInfo.files.map((file) => {
                    file.version = versionInfo.version;
                    return file;
                }));
            }
        });
        return response;
    }
    async downloadDb(response, { version, fileName }) {
        try {
            const dbPath = AssistantPath.getDataBasePath();
            response.download(path.join(dbPath, version, fileName), (err) => {
                if (err) {
                    console.error(`Erro ao realizar download da versão ${fileName}`, err);
                    response.status(500).send({
                        error: `Error ao processar download. Err: ${err.message}`,
                        stack: err.stack,
                        version,
                        fileName,
                    });
                }
            });
        }
        catch (error) {
            response.status(500).send({
                error: error.message,
            });
        }
    }
    getPackageNamesInPage(pageContent, extension = 'tar.gz') {
        const pattern = new RegExp(`(?<=href=")[\\w\\.]+${extension}`, 'gim');
        return pageContent.match(pattern);
    }
    getVersionByPackageName(packageName) {
        const pattern = /[\d\\.]+(?=\.tar\.gz)/i;
        const [result] = packageName.match(pattern) || [];
        return result;
    }
    async loadUpdaterPackages() {
        const packageUrl = Request.removeStartEndBar(process.env.ASSIST_UPDATER_PACKAGE_URL || '');
        const updaterVersionsPath = AssistantPath.getUpdaterVersionPath();
        if (!packageUrl) {
            return Promise.reject(new Error('Parameter {ASSIST_UPDATER_PACKAGES_URL} not defined'));
        }
        try {
            const response = {
                url: packageUrl,
                success: [],
                fail: [],
            };
            const actualVersions = await UpdaterVersion.list();
            const mappedVersions = actualVersions.map(({ version }) => version);
            const packagePage = await Request.get(packageUrl);
            const packageNames = this.getPackageNamesInPage(packagePage) || [];
            const packagesInServer = packageNames.map((name) => {
                const version = this.getVersionByPackageName(name) || '';
                return {
                    name,
                    version,
                    urlToDownload: `${packageUrl}/${name}`,
                };
            });
            const packagesToAdd = packagesInServer.filter(({ version }) => {
                const filePath = path.join(updaterVersionsPath, `${version}.tar.gz`);
                return !mappedVersions.includes(version) || !fs.existsSync(filePath);
            });
            const promiseCollection = packagesToAdd.map(({ urlToDownload, version }) => {
                const downloadPromise = Request.downloadFile(urlToDownload, path.join(updaterVersionsPath, `${version}.tar.gz`));
                return downloadPromise;
            });
            const downloadResult = await Promise.allSettled(promiseCollection);
            const errors = [];
            const databaseResult = [];
            downloadResult.forEach((result) => {
                if (result.status === 'fulfilled') {
                    const version = this.getVersionByPackageName(result.value.fileName) || '';
                    if (!mappedVersions.includes(version)) {
                        databaseResult.push(UpdaterVersion.create({ version }));
                    }
                }
                else {
                    errors.push(result.reason);
                }
            });
            await Promise.allSettled(databaseResult).then((dbResultResponse) => {
                dbResultResponse.forEach((resp) => {
                    if (resp.status === 'fulfilled') {
                        const savedItem = resp.value;
                        response.success.push(savedItem);
                    }
                    else {
                        response.fail.push({ error: true, message: resp.reason });
                    }
                });
            });
            return response;
        }
        catch (error) {
            return Promise.reject(new Error(`Error in download from ${packageUrl}. Error: ${error.message}`));
        }
    }
    encrypt(params) {
        const { text, key } = params;
        return { encrypted: Crypt.encrypt(text, key) };
    }
    async getAgentsVersion({ clientType = '' }) {
        const { version: updater } = await UpdaterVersion.getApprovedVersion();
        const { version: installer } = await AssistantInstaller.getApprovedVersion(clientType);
        return {
            updater,
            installer,
        };
    }
    sendInstallNotification(installInfo) {
        const { info } = installInfo;
        const keyMap = {
            DIR: 'DIRECTORY',
            SRV: 'SERVER',
            FILIAL: 'UNIT',
            LOJA: 'STORE',
            CAIXA: 'CASHER',
            BUILD_TYPE: 'INSTALL_MODE',
        };
        const decodedInfo = Base64.decode(info, false) || '';
        const SYSDATE = format(new Date(), 'dd/MM/yyyy HH:mm');
        const telegramData = decodedInfo.split('\n').reduce((data, line) => {
            const [key = ''] = line.match(/\w+(?==)/g) || [];
            let value = line.substring(key.length + 1).trim();
            const mappedKey = keyMap[key] || key;
            switch (key) {
                case 'CAIXA':
                    if (value) {
                        value = `Caixa: ${value}`;
                    }
                    break;
                case 'BUILD_TYPE':
                    if (value === 'SAAS') {
                        value = 'SaaS';
                    }
                    else {
                        value = 'Hosting / On Premise';
                    }
                    break;
                default: break;
            }
            data[mappedKey] = value;
            return data;
        }, { SYSDATE });
        Telegram.sendNotification('wizard-install', telegramData);
        return {
            status: 'SUCCESS',
        };
    }
}
const instance = new AssistantControl();
export { instance as AssistantControl };
export default instance;
