import { PrismaClient } from '../db/index.js';
import { Gzip } from '../util/index.js';
class ProductSetting {
    encodeConfig(config = {}) {
        return Gzip.encode(JSON.stringify(config, null, 2));
    }
    async create(params) {
        const { devopsId, config } = params;
        const { INTERNAL_PROPS = {} } = config;
        delete config.INTERNAL_PROPS;
        const data = {
            devopsId: Number(devopsId),
            config: this.encodeConfig(config) || '',
            detail: {
                create: {
                    devopsId: Number(devopsId),
                    config: this.encodeConfig(INTERNAL_PROPS) || '',
                },
            },
        };
        const createdResponse = await PrismaClient.productSetting.create({ data, include: { detail: true } });
        const productResponse = await PrismaClient.product.findFirst({
            where: { devopsId: Number(devopsId) },
        });
        createdResponse.config = JSON.parse(Gzip.decode(createdResponse.config));
        return {
            productName: productResponse?.productName,
            ...createdResponse,
        };
    }
    async list(options, filter) {
        const { decodeGZip = true, includeDetail = false, } = options || {};
        const findParams = {
            include: { product: true, detail: includeDetail },
            where: {
                devopsId: filter?.devopsId,
            },
        };
        if (!filter || !filter.devopsId) {
            delete findParams.where.devopsId;
        }
        const data = await PrismaClient.productSetting.findMany(findParams);
        return data.map(({ config, product, detail }) => {
            let configValue = decodeGZip || detail ? JSON.parse(Gzip.decode(config)) : config;
            if (detail) {
                const { INVISIBLE, READ_ONLY, MORE_INFO } = JSON.parse(Gzip.decode(detail.config));
                configValue = Object.keys(configValue).map((paramKey) => {
                    const { label = '', description = '' } = (MORE_INFO || {})[paramKey] || {};
                    return ({
                        name: paramKey,
                        value: configValue[paramKey],
                        label,
                        description,
                        isVisible: !(INVISIBLE || []).includes(paramKey),
                        isEditable: !(READ_ONLY || []).includes(paramKey),
                    });
                }, {});
                if (!decodeGZip) {
                    configValue = Gzip.encode(JSON.stringify(configValue));
                }
            }
            return ({
                config: configValue,
                devopsId: product.devopsId,
                productName: product.productName,
            });
        });
    }
    async find(devopsId, options) {
        const { decodeGZip = true, } = options || {};
        const data = await PrismaClient.productSetting.findUnique({
            where: { devopsId: Number(devopsId) },
            include: {
                detail: true,
            },
        });
        if (decodeGZip && data && data.config) {
            data.config = JSON.parse(Gzip.decode(data.config));
            if (data.detail && data.detail.config) {
                data.detail = JSON.parse(Gzip.decode(data.detail.config));
            }
        }
        return data;
    }
    async remove({ devopsId, productsToRemove }) {
        const products = Array.isArray(productsToRemove) ? productsToRemove : [];
        const where = { devopsId: { in: products.map(Number) } };
        if (devopsId) {
            const response = await PrismaClient.productSetting.deleteMany({
                where: { devopsId: { in: devopsId.toString().split(',').map((id) => Number(id)) } },
            });
            return response;
        }
        if (Array.isArray(productsToRemove)) {
            const response = PrismaClient.productSetting.deleteMany({ where });
            return response;
        }
    }
    saveProductSettingDetail(detailRecord, isNew) {
        const { devopsId, detail, productSettingId, } = detailRecord;
        if (isNew) {
            return PrismaClient.productSettingDetail.create({
                data: {
                    devopsId,
                    config: detail,
                    productSettingId,
                },
            });
        }
        return PrismaClient.productSettingDetail.update({
            where: { devopsId },
            data: { config: detail },
        });
    }
    async modify(params) {
        const { devopsId: devopsIdStr, config } = params;
        const { INTERNAL_PROPS = {} } = config;
        delete config.INTERNAL_PROPS;
        const detail = this.encodeConfig(INTERNAL_PROPS) || '';
        const devopsId = Number(devopsIdStr);
        const savedDetail = await PrismaClient.productSettingDetail.findFirst({
            where: { devopsId },
        });
        const detailRecord = {
            devopsId,
            detail,
            productSettingId: '',
        };
        if (!savedDetail) {
            const prodSetting = await PrismaClient.productSetting.findFirst({
                where: { devopsId },
            });
            if (prodSetting) {
                detailRecord.productSettingId = prodSetting.id;
            }
        }
        const [response] = await PrismaClient.$transaction([
            PrismaClient.productSetting.update({
                where: { devopsId },
                data: { config: this.encodeConfig(config) || '' },
            }),
            this.saveProductSettingDetail(detailRecord, savedDetail === null),
        ]);
        const productResponse = await PrismaClient.product.findFirst({
            where: { devopsId },
        });
        response.config = JSON.parse(Gzip.decode(response.config));
        return {
            productName: productResponse?.productName,
            ...response,
        };
    }
    async getSettingForApi(devopsId) {
        const options = {
            decodeGZip: false,
            includeDetail: true,
        };
        const filter = {
            devopsId: devopsId ? Number(devopsId) : undefined,
        };
        const productSetting = await this.list(options, filter);
        return productSetting;
    }
}
const instance = new ProductSetting();
export { instance as ProductSetting };
export default instance;
